Package: simhelpers
Type: Package
Title: Helper Functions for Simulation Studies
Version: 0.3.1
Authors@R: c(
    person(given = "Megha", 
           family = "Joshi", 
           email = "megha.j456@gmail.com", 
           role = c("aut", "cre"),
           comment = c(ORCID = "0000-0001-7936-076X")),
    person(given = "James", 
           family = "Pustejovsky", 
           email = "jepusto@gmail.com", 
           role = c("aut"), 
           comment = c(ORCID = "0000-0003-0591-9465")))
Description: Calculates performance criteria measures and associated Monte Carlo standard errors for simulation results. Includes functions to help run simulation studies, following a general simulation workflow that closely aligns with the approach described by Morris, White, and Crowther (2019) <DOI:10.1002/sim.8086>. Also includes functions for calculating bootstrap confidence intervals (including normal, basic, studentized, percentile, bias-corrected, and bias-corrected-and-accelerated) with tidy output, as well as for extrapolating confidence interval coverage rates and hypothesis test rejection rates following techniques suggested by Boos and Zhang (2000) <DOI:10.1080/01621459.2000.10474226>.
URL: https://meghapsimatrix.github.io/simhelpers/
BugReports: https://github.com/meghapsimatrix/simhelpers/issues
Depends: R (>= 2.10)
License: GPL-3
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
SystemRequirements: RStudio
Imports: stats, furrr, tidyr, rstudioapi, Rdpack
Suggests: dplyr, tibble, purrr, future, knitr, rmarkdown, pkgdown,
        covr, testthat, kableExtra, ggplot2, broom, boot
RdMacros: Rdpack
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2025-01-10 18:03:35 UTC; meghajoshi
Author: Megha Joshi [aut, cre] (<https://orcid.org/0000-0001-7936-076X>),
  James Pustejovsky [aut] (<https://orcid.org/0000-0003-0591-9465>)
Maintainer: Megha Joshi <megha.j456@gmail.com>
Repository: CRAN
Date/Publication: 2025-01-10 21:50:02 UTC
Built: R 4.6.0; ; 2026-04-26 21:38:39 UTC; unix
