# Generated by roxygen2: do not edit by hand

export(bootstrap_CIs)
export(bootstrap_pvals)
export(bundle_sim)
export(calc_absolute)
export(calc_coverage)
export(calc_rejection)
export(calc_relative)
export(calc_relative_var)
export(create_skeleton)
export(evaluate_by_row)
export(extrapolate_coverage)
export(extrapolate_rejection)
export(repeat_and_stack)
importFrom(Rdpack,reprompt)
importFrom(stats,qnorm)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,var)
